﻿using System;
using System.Diagnostics;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스톱워치
        /// </summary>
        private Stopwatch stopwatch = new Stopwatch();

        /// <summary>
        /// 멀티미디어 타이머
        /// </summary>
        private MultimediaTimer multimediaTimer;

        /// <summary>
        /// 표시 타이머
        /// </summary>
        private Timer displayTimer;

        /// <summary>
        /// 현재 경과 시간 (단위 : 밀리초)
        /// </summary>
        private double currentElapsedTime;

        /// <summary>
        /// 이전 시간 (단위 : 밀리초)
        /// </summary>
        private double previousTime;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.multimediaTimer = new MultimediaTimer();

            this.multimediaTimer.Interval   = 1;
            this.multimediaTimer.Resolution = 0;

            this.displayTimer = new Timer();

            this.displayTimer.Interval = 30;

            FormClosing               += Form_FormClosing;
            this.multimediaTimer.Tick += multimediaTimer_Tick;
            this.displayTimer.Tick    += displayTimer_Tick;

            this.multimediaTimer.Start();
            this.displayTimer.Start();

            this.stopwatch.Start();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 폼을 닫을 경우 처리하기 - Form_FormClosing(sender, e)

        /// <summary>
        /// 폼을 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.displayTimer.Stop();
            this.multimediaTimer.Stop();
        }

        #endregion
        #region 멀티미디어 타이머 틱 처리하기 - multimediaTimer_Tick(sender, e)

        /// <summary>
        /// 멀티미디어 타이머 틱 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void multimediaTimer_Tick(object sender, EventArgs e)
        {
            this.currentElapsedTime = this.stopwatch.ElapsedMilliseconds - this.previousTime;

            Console.WriteLine($"멀티미디어 타이머 : [{DateTime.UtcNow:o}] {currentElapsedTime}");

            this.previousTime = this.stopwatch.ElapsedMilliseconds ;
        }

        #endregion
        #region 표시 타이머 틱 처리하기 - displayTimer_Tick(sender, e)

        /// <summary>
        /// 표시 타이머 틱 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            this.currentElaspesTimeValueLabel.Text = this.currentElapsedTime.ToString();
        }

        #endregion
    }
}